"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fnProgressHandler = fnProgressHandler;
exports.fnCheckWebpackConfig = fnCheckWebpackConfig;
exports.fnGetValueByPath = fnGetValueByPath;
exports.fnBuildCopyFiles = fnBuildCopyFiles;
exports.fnBuildExternals = fnBuildExternals;
exports.fnBuild4DevelopmentENV = fnBuild4DevelopmentENV;
exports.fnBuild4ProductionENV = fnBuild4ProductionENV;
exports.fnBuildHTMLData = fnBuildHTMLData;
exports.fnBuildHTML = fnBuildHTML;
exports.createDomain = createDomain;
exports.fnGetNode = fnGetNode;
exports.fnBuildSourceMap = fnBuildSourceMap;
exports.fnBuildTemplateParametersWithScripts = fnBuildTemplateParametersWithScripts;
exports.getOptions = getOptions;
Object.defineProperty(exports, "notifier", {
  enumerable: true,
  get: function get() {
    return _nodeNotifier.default;
  }
});
Object.defineProperty(exports, "chalk", {
  enumerable: true,
  get: function get() {
    return _chalk.default;
  }
});

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _objectSpread3 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _nodeNotifier = _interopRequireDefault(require("node-notifier"));

var _chalk = _interopRequireDefault(require("chalk"));

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

/**************************************************
 * Created by nanyuantingfeng on 11/06/2017 10:21.
 **************************************************/
function fnProgressHandler(percent, msg1, msg2) {
  var stream = process.stdout;

  if (stream.isTTY && percent < 0.7) {
    stream.cursorTo(0);
    stream.write("\u231B  ".concat(_chalk.default.magenta(msg2), " ").concat(msg1));
    stream.clearLine(1);
  } else if (percent >= 1) {
    console.log(_chalk.default.green('\nwebpack: bundle build is now finished'));
  }
}

function fnCheckWebpackConfig(webpackConfig) {
  var configs = Array.isArray(webpackConfig) ? webpackConfig : [webpackConfig];
  var hasEmptyEntry = configs.some(function (c) {
    return Object.keys(c.entry || {}).length === 0;
  });

  if (hasEmptyEntry) {
    var e = new Error('no webpack entry found');
    e.name = 'NoEntry';
    throw e;
  }
}

function fnGetValueByPath(path) {
  return !_fs.default.existsSync(path) ? {} : require(path);
}

function fnBuildCopyFiles(files) {
  if (Array.isArray(files)) {
    return files.map(function (from) {
      return {
        from: from
      };
    });
  }

  return Object.keys(files).filter(function (key) {
    var file = files[key];
    return typeof file === 'string' || !!files[key].path;
  }).map(function (key) {
    var file = files[key];

    if (typeof file === 'string') {
      return {
        from: file
      };
    }

    return {
      from: file.path,
      to: file.to
    };
  });
}

function fnBuildExternals(files) {
  var ret = {};
  Object.keys(files).forEach(function (key) {
    var file = files[key];

    if (typeof file === 'string') {
      ret[key] = file;
    } else if (file.name) {
      ret[key] = file.name;
    }
  });
  return ret;
}

function fnBuild4DevelopmentENV(filesMap) {
  var ret = [];
  Object.keys(filesMap).forEach(function (key) {
    var line = filesMap[key];
    var path = line.path;

    if (line.name && path) {
      ret.push(path);
    }
  });
  return ret;
}

function fnBuild4ProductionENV(filesMap) {
  var ret = [];
  Object.keys(filesMap).forEach(function (key) {
    var line = filesMap[key];
    var path = line.path;

    if (line.name && path) {
      var paths = path.split('/');
      path = paths[paths.length - 1];
      ret.push(path);
    }
  });
  return ret;
}

function fnBuildHTMLData(filesMap, env) {
  switch (env) {
    case 'production':
    case 'beta':
      return fnBuild4ProductionENV(filesMap);

    default:
      return fnBuild4DevelopmentENV(filesMap);
  }
}

function fnBuildHTML(context, env) {
  var _context$externals = context.externals,
      externals = _context$externals === void 0 ? {} : _context$externals,
      _context$sdks = context.sdks,
      sdks = _context$sdks === void 0 ? {} : _context$sdks,
      DLL_FILENAME = context.DLL_FILENAME,
      htmlWebpackPluginOptions = context.htmlWebpackPluginOptions;
  var entry = context.entry || context.packageMap.entry;

  if (!entry) {
    throw new Error('entry is an invalid value');
  }

  if (typeof entry === 'string') {
    context.entry = entry = {
      index: entry
    };
  }

  var paths = fnBuildHTMLData(externals, env);
  var entryNames = Object.keys(entry);
  var options = (0, _objectSpread3.default)({
    template: _path.default.join(__dirname, '../assets/index.hbs'),
    favicon: _path.default.join(__dirname, '../assets/favicon.ico')
  }, htmlWebpackPluginOptions);
  return entryNames.map(function (entryName) {
    var sdk = sdks[entryName];

    if (typeof sdk === 'string') {
      sdk = [sdk];
    }

    var scripts = paths.slice(0);

    if (sdk) {
      scripts.push.apply(scripts, (0, _toConsumableArray2.default)(sdk));
    }

    if (DLL_FILENAME) {
      scripts.push(DLL_FILENAME);
    }

    return (0, _objectSpread3.default)({
      PATHS: scripts,
      scripts: scripts,
      entryName: entryName,
      filename: "".concat(entryName, ".html"),
      chunks: [entryName],
      chunksSortMode: 'dependency',
      inject: true,
      templateParameters: fnBuildTemplateParametersWithScripts(scripts)
    }, options);
  });
}

function createDomain(_ref) {
  var host = _ref.host,
      port = _ref.port;
  return "http://".concat(host, ":").concat(port);
}

function fnGetNode(packageMap) {
  var emptyBuildIns = ['child_process', 'cluster', 'dgram', 'dns', 'fs', 'module', 'net', 'readline', 'repl', 'tls'];
  var browser = packageMap.browser || {};
  return emptyBuildIns.reduce(function (obj, name) {
    if (!(name in browser)) {
      return (0, _objectSpread3.default)({}, obj, (0, _defineProperty2.default)({}, name, 'empty'));
    }

    return obj;
  }, {});
}

function fnBuildSourceMap() {
  var devtool = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
  var ENV = arguments.length > 1 ? arguments[1] : undefined;

  /******************
   *#source-map 编译过慢
   * production 环境不需要
   * beta 环境需要
   */
  if (devtool === true) {
    devtool = ENV.isProduction ? false : ENV.isDevelopment ? '#cheap-module-source-map' : ENV.isBeta ? '#cheap-module-source-map' : false;
  }

  return devtool;
}

function fnBuildTemplateParametersWithScripts(scripts) {
  return function (compilation, assets, options) {
    var entryName = options.entryName;
    var stats = compilation.getStats().toJson();
    var currentAssets = stats.entrypoints[entryName].assets;
    var js = currentAssets.filter(function (n) {
      return _path.default.extname(n) === '.js';
    });
    var css = currentAssets.filter(function (n) {
      return _path.default.extname(n) === '.css';
    });
    assets.js = unique(scripts.concat(assets.js).concat(js));
    assets.css = unique(assets.css.concat(css));
    return {
      compilation: compilation,
      webpack: compilation.getStats().toJson(),
      webpackConfig: compilation.options,
      htmlWebpackPlugin: {
        files: assets,
        options: options,
        scripts: js,
        styles: css
      }
    };
  };
}

function unique(array) {
  return array.filter(function (item, index, array) {
    return array.indexOf(item) === index;
  });
}

function getOptions(options) {
  var defaultOptions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  return options === true ? defaultOptions : options;
}